"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const EntityManager_1 = require("./EntityManager");
const metadata_1 = require("./metadata");
const utils_1 = require("./utils");
class MikroORM {
    static async init(options) {
        const orm = new MikroORM(options);
        const driver = await orm.connect();
        orm.em = new EntityManager_1.EntityManager(orm.config, driver);
        try {
            const storage = new metadata_1.MetadataDiscovery(orm.em, orm.config, orm.logger);
            await storage.discover();
            return orm;
        }
        catch (e) {
            await orm.close(true);
            throw e;
        }
    }
    constructor(options) {
        if (options instanceof utils_1.Configuration) {
            this.config = options;
        }
        else {
            this.config = new utils_1.Configuration(options);
        }
        this.driver = this.config.getDriver();
        this.logger = this.config.getLogger();
    }
    async connect() {
        await this.driver.getConnection().connect();
        const clientUrl = this.config.getClientUrl(true);
        const dbName = this.config.get('dbName');
        this.logger.info(`MikroORM: successfully connected to database ${dbName}${clientUrl ? ' on ' + clientUrl : ''}`);
        return this.driver;
    }
    async isConnected() {
        return this.driver.getConnection().isConnected();
    }
    async close(force = false) {
        return this.driver.getConnection().close(force);
    }
}
exports.MikroORM = MikroORM;
