"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
class FileCacheAdapter {
    constructor(options) {
        this.options = options;
        if (!fs_1.existsSync(this.options.cacheDir)) {
            fs_1.mkdirSync(this.options.cacheDir);
        }
    }
    get(name) {
        if (!fs_1.existsSync(this.path(name))) {
            return null;
        }
        const buffer = fs_1.readFileSync(this.path(name));
        const payload = JSON.parse(buffer.toString());
        if (!fs_1.existsSync(payload.origin) || payload.modified !== this.getModifiedTime(payload.origin)) {
            return null;
        }
        return payload.data;
    }
    set(name, data, origin) {
        const payload = {
            modified: this.getModifiedTime(origin),
            data,
            origin,
        };
        fs_1.writeFileSync(this.path(name), JSON.stringify(payload));
    }
    path(name) {
        return `${this.options.cacheDir}/${name}.json`;
    }
    getModifiedTime(origin) {
        return fs_1.statSync(origin).mtimeMs;
    }
}
exports.FileCacheAdapter = FileCacheAdapter;
