import { Configuration } from '../utils';
export declare abstract class Connection {
    protected readonly config: Configuration;
    protected readonly logger: import("../utils").Logger;
    constructor(config: Configuration);
    /**
     * Establishes connection to database
     */
    abstract connect(): Promise<void>;
    /**
     * Are we connected to the database
     */
    abstract isConnected(): Promise<boolean>;
    /**
     * Closes the database connection (aka disconnect)
     */
    abstract close(force?: boolean): Promise<void>;
    /**
     * Returns default client url for given driver (e.g. mongodb://localhost:27017 for mongodb)
     */
    abstract getDefaultClientUrl(): string;
    /**
     * Begins a transaction (if supported)
     */
    beginTransaction(savepoint?: string): Promise<void>;
    /**
     * Commits statements in a transaction
     */
    commit(savepoint?: string): Promise<void>;
    /**
     * Rollback changes in a transaction
     */
    rollback(savepoint?: string): Promise<void>;
    abstract execute(query: string, params?: any[], method?: string): Promise<QueryResult | any | any[]>;
    protected logQuery(query: string): void;
}
export interface QueryResult {
    affectedRows: number;
    insertId: number;
}
