"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const promise_1 = require("mysql2/promise");
const fs_1 = require("fs");
const url_1 = require("url");
const Connection_1 = require("./Connection");
class MySqlConnection extends Connection_1.Connection {
    async connect() {
        this.connection = await promise_1.createConnection(this.getConnectionOptions());
    }
    async close(force) {
        await this.connection.end({ force });
    }
    async isConnected() {
        try {
            await this.connection.query('SELECT 1');
            return true;
        }
        catch (_a) {
            return false;
        }
    }
    async beginTransaction() {
        await this.query('START TRANSACTION');
    }
    async commit() {
        await this.query('COMMIT');
    }
    async rollback() {
        await this.query('ROLLBACK');
    }
    getDefaultClientUrl() {
        return 'mysql://root@127.0.0.1:3306';
    }
    async execute(query, params = [], method = 'all') {
        try {
            const now = Date.now();
            const res = await this.connection.execute(query, params);
            this.logQuery(query + ` [took ${Date.now() - now} ms]`);
            if (method === 'get') {
                return res[0][0];
            }
            return res[0];
        }
        catch (e) {
            e.message += `\n in query: ${query}`;
            if (params && params.length) {
                e.message += `\n with params: ${JSON.stringify(params)}`;
            }
            throw e;
        }
    }
    getConnectionOptions() {
        const ret = {};
        const url = new url_1.URL(this.config.getClientUrl());
        ret.host = this.config.get('host', url.hostname);
        ret.port = this.config.get('port', +url.port);
        ret.user = this.config.get('user', url.username);
        ret.password = this.config.get('password', url.password);
        ret.database = this.config.get('dbName', url.pathname.replace(/^\//, ''));
        if (this.config.get('multipleStatements')) {
            ret.multipleStatements = this.config.get('multipleStatements');
        }
        return ret;
    }
    async loadFile(path) {
        const file = fs_1.readFileSync(path);
        await this.query(file.toString());
    }
    async query(sql) {
        const now = Date.now();
        await this.connection.query(sql);
        this.logQuery(`${sql} [took ${Date.now() - now} ms]`);
    }
}
exports.MySqlConnection = MySqlConnection;
