import { EntityData, IEntityType, IPrimaryKey } from '../decorators';
import { DatabaseDriver } from './DatabaseDriver';
import { Connection } from '../connections/Connection';
import { FilterQuery } from './IDatabaseDriver';
import { QueryBuilder, QueryOrder } from '../query';
export declare abstract class AbstractSqlDriver<C extends Connection> extends DatabaseDriver<C> {
    find<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T>, populate?: string[], orderBy?: Record<string, QueryOrder>, limit?: number, offset?: number): Promise<T[]>;
    findOne<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T> | string, populate?: string[]): Promise<T | null>;
    count(entityName: string, where: any): Promise<number>;
    nativeInsert<T extends IEntityType<T>>(entityName: string, data: EntityData<T>): Promise<number>;
    nativeUpdate<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T>, data: EntityData<T>): Promise<number>;
    nativeDelete<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T> | string | any): Promise<number>;
    protected createQueryBuilder(entityName: string): QueryBuilder;
    protected extractManyToMany<T extends IEntityType<T>>(entityName: string, data: EntityData<T>): EntityData<T>;
    protected processManyToMany<T extends IEntityType<T>>(entityName: string, pk: IPrimaryKey, collections: EntityData<T>): Promise<void>;
}
