import { DriverConfig, FilterQuery, IDatabaseDriver } from './IDatabaseDriver';
import { EntityData, EntityMetadata, EntityProperty, IEntity, IEntityType, IPrimaryKey } from '../decorators';
import { Connection } from '../connections/Connection';
import { Configuration } from '../utils';
import { QueryOrder } from '../query';
export declare abstract class DatabaseDriver<C extends Connection> implements IDatabaseDriver<C> {
    protected readonly config: Configuration;
    protected readonly connection: Connection;
    protected readonly metadata: Record<string, EntityMetadata<any>>;
    protected readonly logger: import("../utils").Logger;
    protected transactionLevel: number;
    protected transactionRolledBack: boolean;
    constructor(config: Configuration);
    abstract find<T extends IEntity>(entityName: string, where: FilterQuery<T>, populate?: string[], orderBy?: Record<string, QueryOrder>, limit?: number, offset?: number): Promise<T[]>;
    abstract findOne<T extends IEntity>(entityName: string, where: FilterQuery<T> | string, populate: string[]): Promise<T | null>;
    abstract nativeInsert<T extends IEntity>(entityName: string, data: EntityData<T>): Promise<IPrimaryKey>;
    abstract nativeUpdate<T extends IEntity>(entityName: string, where: FilterQuery<IEntity> | IPrimaryKey, data: EntityData<T>): Promise<number>;
    abstract nativeDelete<T extends IEntity>(entityName: string, where: FilterQuery<IEntity> | IPrimaryKey): Promise<number>;
    abstract count<T extends IEntity>(entityName: string, where: FilterQuery<T>): Promise<number>;
    aggregate(entityName: string, pipeline: any[]): Promise<any[]>;
    loadFromPivotTable<T extends IEntity>(prop: EntityProperty, owners: IPrimaryKey[]): Promise<Record<string, T[]>>;
    normalizePrimaryKey<T = number | string>(data: IPrimaryKey): T;
    denormalizePrimaryKey(data: number | string): IPrimaryKey;
    mapResult<T extends IEntityType<T>>(result: T, meta: EntityMetadata): T;
    getConnection(): C;
    beginTransaction(): Promise<void>;
    commit(): Promise<void>;
    rollback(): Promise<void>;
    transactional(cb: () => Promise<any>): Promise<any>;
    isInTransaction(): boolean;
    getConfig(): DriverConfig;
    private runTransaction;
    private getSavePointName;
}
