import { FilterQuery, ObjectID } from 'mongodb';
import { DatabaseDriver } from './DatabaseDriver';
import { MongoConnection } from '../connections/MongoConnection';
import { EntityData, IEntityType, IPrimaryKey } from '../decorators';
import { QueryOrder } from '../query';
import { DriverConfig } from './IDatabaseDriver';
export declare class MongoDriver extends DatabaseDriver<MongoConnection> {
    protected readonly connection: MongoConnection;
    find<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T>, populate: string[], orderBy: Record<string, QueryOrder>, limit: number, offset: number): Promise<T[]>;
    findOne<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T> | IPrimaryKey, populate?: string[]): Promise<T | null>;
    count<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T>): Promise<number>;
    nativeInsert<T extends IEntityType<T>>(entityName: string, data: EntityData<T>): Promise<ObjectID>;
    nativeUpdate<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T> | IPrimaryKey, data: EntityData<T>): Promise<number>;
    nativeDelete<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T> | IPrimaryKey): Promise<number>;
    aggregate(entityName: string, pipeline: any[]): Promise<any[]>;
    normalizePrimaryKey<T = number | string>(data: IPrimaryKey | ObjectID): T;
    denormalizePrimaryKey(data: number | string): IPrimaryKey;
    getConfig(): DriverConfig;
    private renameFields;
}
