"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const entity_1 = require("../entity");
const metadata_1 = require("../metadata");
class ArrayCollection {
    constructor(owner, items) {
        this.owner = owner;
        this.items = [];
        if (items) {
            this.items = items;
            Object.assign(this, items);
        }
    }
    getItems() {
        return [...this.items];
    }
    toArray(parent = this.owner) {
        return this.getItems().map(item => entity_1.EntityTransformer.toObject(item, parent, true));
    }
    getIdentifiers(field = 'id') {
        return this.getItems().map(i => i[field]);
    }
    add(...items) {
        for (const item of items) {
            if (!this.contains(item)) {
                this.handleInverseSide(item, 'add');
                this.items.push(item);
            }
        }
        Object.assign(this, items);
    }
    set(items) {
        this.removeAll();
        this.add(...items);
    }
    remove(...items) {
        for (const item of items) {
            this.handleInverseSide(item, 'remove');
            const idx = this.items.findIndex(i => i.id === item.id);
            if (idx !== -1) {
                delete this[this.items.length]; // remove last item
                this.items.splice(idx, 1);
                Object.assign(this, this.items); // reassign array access
            }
        }
    }
    removeAll() {
        this.remove(...this.items);
    }
    contains(item) {
        return !!this.items.find(i => i === item || !!(i.id && item.id && i.id === item.id));
    }
    count() {
        return this.items.length;
    }
    get length() {
        return this.count();
    }
    *[Symbol.iterator]() {
        for (const item of this.items) {
            yield item;
        }
    }
    handleInverseSide(item, method) {
        if (this.property.owner && this.property.inversedBy && item[this.property.inversedBy].isInitialized()) {
            item[this.property.inversedBy][method](this.owner);
        }
    }
    get property() {
        if (!this._property) {
            const meta = metadata_1.MetadataStorage.getMetadata(this.owner.constructor.name);
            const field = Object.keys(meta.properties).find(k => this.owner[k] === this);
            this._property = meta.properties[field];
        }
        return this._property;
    }
}
exports.ArrayCollection = ArrayCollection;
