"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const EntityFactory_1 = require("./EntityFactory");
const utils_1 = require("../utils");
const metadata_1 = require("../metadata");
const enums_1 = require("./enums");
class EntityAssigner {
    static assign(entity, data) {
        const metadata = metadata_1.MetadataStorage.getMetadata();
        const meta = metadata[entity.constructor.name];
        const props = meta.properties;
        Object.keys(data).forEach(prop => {
            const value = data[prop];
            if (props[prop] && props[prop].reference === enums_1.ReferenceType.MANY_TO_ONE && value) {
                return EntityAssigner.assignReference(entity, value, props[prop], entity.__em);
            }
            const isCollection = props[prop] && [enums_1.ReferenceType.ONE_TO_MANY, enums_1.ReferenceType.MANY_TO_MANY].includes(props[prop].reference);
            if (isCollection && Array.isArray(value)) {
                return EntityAssigner.assignCollection(entity, value, props[prop], entity.__em);
            }
            if (props[prop] && props[prop].reference === enums_1.ReferenceType.SCALAR && EntityFactory_1.SCALAR_TYPES.includes(props[prop].type)) {
                entity[prop] = entity.__em.getValidator().validateProperty(props[prop], value, entity);
            }
            entity[prop] = value;
        });
    }
    static assignReference(entity, value, prop, em) {
        if (utils_1.Utils.isEntity(value)) {
            entity[prop.name] = value;
            return;
        }
        const id = utils_1.Utils.extractPK(value);
        if (id) {
            const normalized = em.getDriver().normalizePrimaryKey(id);
            entity[prop.name] = em.getReference(prop.type, normalized);
            return;
        }
        const name = entity.constructor.name;
        throw new Error(`Invalid reference value provided for '${name}.${prop.name}' in ${name}.assign(): ${JSON.stringify(value)}`);
    }
    static assignCollection(entity, value, prop, em) {
        const invalid = [];
        const items = value.map((item) => {
            if (utils_1.Utils.isEntity(item)) {
                return item;
            }
            if (utils_1.Utils.isPrimaryKey(item)) {
                const id = em.getDriver().normalizePrimaryKey(item);
                return em.getReference(prop.type, id);
            }
            invalid.push(item);
            return item;
        });
        if (invalid.length > 0) {
            const name = entity.constructor.name;
            throw new Error(`Invalid collection values provided for '${name}.${prop.name}' in ${name}.assign(): ${JSON.stringify(invalid)}`);
        }
        entity[prop.name].set(items);
    }
}
exports.EntityAssigner = EntityAssigner;
