"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const uuid_1 = require("uuid");
const EntityTransformer_1 = require("./EntityTransformer");
const EntityAssigner_1 = require("./EntityAssigner");
class EntityHelper {
    static async init(entity, populated = true) {
        await entity.__em.findOne(entity.constructor.name, entity.id);
        entity.populated(populated);
        return entity;
    }
    static decorate(meta, em) {
        const pk = meta.properties[meta.primaryKey];
        if (pk.name === '_id' && !meta.properties['id']) {
            EntityHelper.defineIdProperty(meta, em);
        }
        EntityHelper.defineBaseProperties(meta, em);
        if (!meta.prototype.assign) { // assign can be overridden
            meta.prototype.assign = function (data) {
                EntityAssigner_1.EntityAssigner.assign(this, data);
            };
        }
        if (!meta.prototype.toJSON) { // toJSON can be overridden
            meta.prototype.toJSON = function () {
                return EntityTransformer_1.EntityTransformer.toObject(this);
            };
        }
    }
    /**
     * defines magic id property getter/setter if PK property is `_id` and there is no `id` property defined
     */
    static defineIdProperty(meta, em) {
        Object.defineProperty(meta.prototype, 'id', {
            get() {
                return this._id ? em.getDriver().normalizePrimaryKey(this._id) : null;
            },
            set(id) {
                this._id = id ? em.getDriver().denormalizePrimaryKey(id) : null;
            },
        });
    }
    static defineBaseProperties(meta, em) {
        Object.defineProperties(meta.prototype, {
            __populated: { value: false, writable: true },
            __entity: { value: true },
            __em: { value: em },
            uuid: {
                get() {
                    if (!this.__uuid) {
                        Object.defineProperty(this, '__uuid', { value: uuid_1.v4() });
                    }
                    return this.__uuid;
                },
            },
        });
        meta.prototype.isInitialized = function () {
            return this.__initialized !== false;
        };
        meta.prototype.populated = function (populated = true) {
            this.__populated = populated;
        };
        meta.prototype.toObject = function (parent, isCollection) {
            return EntityTransformer_1.EntityTransformer.toObject(this, parent, isCollection);
        };
        meta.prototype.init = async function (populated = true) {
            return EntityHelper.init(this, populated);
        };
    }
}
exports.EntityHelper = EntityHelper;
