import { EntityManager, FindOptions } from '../EntityManager';
import { EntityData, EntityName, IEntity, IEntityType, IPrimaryKey } from '../decorators';
import { QueryOrder } from '../query';
import { FilterQuery } from '..';
export declare class EntityRepository<T extends IEntityType<T>> {
    private readonly _em;
    protected readonly entityName: EntityName<T>;
    constructor(_em: EntityManager, entityName: EntityName<T>);
    persist(entity: T | IEntity[], flush?: boolean): Promise<void>;
    persistAndFlush(entity: IEntity | IEntity[]): Promise<void>;
    persistLater(entity: IEntity | IEntity[]): void;
    findOne(where: FilterQuery<T> | IPrimaryKey, populate?: string[]): Promise<T | null>;
    find(where: FilterQuery<T> | IPrimaryKey, options?: FindOptions): Promise<T[]>;
    find(where: FilterQuery<T> | IPrimaryKey, populate?: string[], orderBy?: Record<string, QueryOrder>, limit?: number, offset?: number): Promise<T[]>;
    findAll(options?: FindOptions): Promise<T[]>;
    findAll(populate?: string[], orderBy?: Record<string, QueryOrder>, limit?: number, offset?: number): Promise<T[]>;
    remove(where: T | FilterQuery<T> | IPrimaryKey, flush?: boolean): Promise<number>;
    removeAndFlush(entity: IEntity): Promise<void>;
    removeLater(entity: IEntity): void;
    flush(): Promise<void>;
    nativeInsert(data: EntityData<T>): Promise<IPrimaryKey>;
    nativeUpdate(where: FilterQuery<T>, data: EntityData<T>): Promise<number>;
    nativeDelete(where: FilterQuery<T> | any): Promise<number>;
    aggregate(pipeline: any[]): Promise<any[]>;
    /**
     * Gets a reference to the entity identified by the given type and identifier without actually loading it, if the entity is not yet loaded
     */
    getReference(id: IPrimaryKey): T;
    canPopulate(property: string): boolean;
    /**
     * Creates new instance of given entity and populates it with given data
     */
    create(data: EntityData<T>): T;
    count(where?: FilterQuery<T>): Promise<number>;
    protected readonly em: EntityManager;
}
