"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../utils");
const ArrayCollection_1 = require("./ArrayCollection");
class EntityTransformer {
    static toObject(entity, parent, isCollection = false) {
        const ret = (entity.id ? { id: entity.id } : {});
        if (!entity.isInitialized() && entity.id) {
            return ret;
        }
        Object.keys(entity)
            .filter(prop => prop !== 'id' && !prop.startsWith('_'))
            .map(prop => [prop, EntityTransformer.processProperty(prop, entity, parent || entity, isCollection)])
            .filter(([prop, value]) => !!value)
            .forEach(([prop, value]) => ret[prop] = value);
        return ret;
    }
    static processProperty(prop, entity, parent, isCollection) {
        if (entity[prop] instanceof ArrayCollection_1.ArrayCollection) {
            return EntityTransformer.processCollection(prop, entity);
        }
        if (utils_1.Utils.isEntity(entity[prop])) {
            return EntityTransformer.processEntity(prop, entity, parent, isCollection);
        }
        return entity[prop];
    }
    static processEntity(prop, entity, parent, isCollection) {
        const child = entity[prop];
        if (child.isInitialized() && child.__populated && !isCollection && child !== parent) {
            return EntityTransformer.toObject(child, entity);
        }
        return entity[prop].id;
    }
    static processCollection(prop, entity) {
        const col = entity[prop];
        if (col.isInitialized(true) && col.shouldPopulate()) {
            return col.toArray(entity);
        }
        if (col.isInitialized()) {
            return col.getIdentifiers();
        }
    }
}
exports.EntityTransformer = EntityTransformer;
