"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Hydrator_1 = require("./Hydrator");
const entity_1 = require("../entity");
const utils_1 = require("../utils");
class ObjectHydrator extends Hydrator_1.Hydrator {
    hydrateProperty(entity, prop, value) {
        if (prop.reference === entity_1.ReferenceType.MANY_TO_ONE) {
            this.hydrateManyToOne(value, entity, prop);
        }
        else if (prop.reference === entity_1.ReferenceType.ONE_TO_MANY) {
            this.hydrateOneToMany(entity, prop, value);
        }
        else if (prop.reference === entity_1.ReferenceType.MANY_TO_MANY) {
            this.hydrateManyToMany(entity, prop, value);
        }
        else if (value) { // ReferenceType.SCALAR
            this.hydrateScalar(entity, prop, value);
        }
    }
    hydrateOneToMany(entity, prop, value) {
        entity[prop.name] = new entity_1.Collection(entity, undefined, !!value);
    }
    hydrateScalar(entity, prop, value) {
        entity[prop.name] = value;
    }
    hydrateManyToMany(entity, prop, value) {
        if (prop.owner) {
            return this.hydrateManyToManyOwner(entity, prop, value);
        }
        else if (!entity[prop.name]) {
            this.hydrateManyToManyInverse(entity, prop);
        }
    }
    hydrateManyToManyOwner(entity, prop, value) {
        if (Array.isArray(value)) {
            const items = value.map((id) => this.factory.createReference(prop.type, this.driver.normalizePrimaryKey(id)));
            entity[prop.name] = new entity_1.Collection(entity, items);
        }
        else if (!entity[prop.name]) {
            const items = this.driver.getConfig().usesPivotTable ? undefined : [];
            entity[prop.name] = new entity_1.Collection(entity, items, false);
        }
    }
    hydrateManyToManyInverse(entity, prop) {
        if (!entity[prop.name]) {
            entity[prop.name] = new entity_1.Collection(entity, undefined, false);
        }
    }
    hydrateManyToOne(value, entity, prop) {
        if (value && !utils_1.Utils.isEntity(value)) {
            const id = this.driver.normalizePrimaryKey(value);
            entity[prop.name] = this.factory.createReference(prop.type, id);
        }
    }
}
exports.ObjectHydrator = ObjectHydrator;
