"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../utils");
const metadata_1 = require("../metadata");
const entity_1 = require("../entity");
class ChangeSetComputer {
    constructor(validator, originalEntityData, identifierMap) {
        this.validator = validator;
        this.originalEntityData = originalEntityData;
        this.identifierMap = identifierMap;
        this.metadata = metadata_1.MetadataStorage.getMetadata();
    }
    computeChangeSet(entity) {
        const changeSet = { entity };
        const meta = this.metadata[entity.constructor.name];
        changeSet.name = meta.name;
        changeSet.collection = meta.collection;
        changeSet.payload = this.computePayload(entity);
        this.validator.validate(changeSet.entity, changeSet.payload, meta);
        for (const prop of Object.values(meta.properties)) {
            this.processReference(changeSet, prop);
        }
        if (entity.id && Object.keys(changeSet.payload).length === 0) {
            return null;
        }
        return changeSet;
    }
    computePayload(entity) {
        if (entity.id && this.originalEntityData[entity.uuid]) {
            return utils_1.Utils.diffEntities(this.originalEntityData[entity.uuid], entity);
        }
        else {
            return utils_1.Utils.prepareEntity(entity);
        }
    }
    processReference(changeSet, prop) {
        if (prop.reference === entity_1.ReferenceType.MANY_TO_MANY && prop.owner) {
            this.processManyToMany(changeSet, prop, changeSet.entity[prop.name]);
        }
        else if (prop.reference === entity_1.ReferenceType.MANY_TO_ONE && changeSet.entity[prop.name]) {
            this.processManyToOne(prop, changeSet);
        }
    }
    processManyToOne(prop, changeSet) {
        const pk = this.metadata[prop.type].primaryKey;
        const entity = changeSet.entity[prop.name];
        if (!entity[pk]) {
            changeSet.payload[prop.name] = this.identifierMap[entity.uuid];
        }
    }
    processManyToMany(changeSet, prop, collection) {
        if (prop.owner && collection.isDirty()) {
            const pk = this.metadata[prop.type].primaryKey;
            changeSet.payload[prop.name] = collection.getItems().map(item => item[pk] || this.identifierMap[item.uuid]);
            collection.setDirty(false);
        }
    }
}
exports.ChangeSetComputer = ChangeSetComputer;
