"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const metadata_1 = require("../metadata");
const entity_1 = require("../entity");
class ChangeSetPersister {
    constructor(driver, identifierMap) {
        this.driver = driver;
        this.identifierMap = identifierMap;
        this.metadata = metadata_1.MetadataStorage.getMetadata();
    }
    async persistToDatabase(changeSet) {
        const meta = this.metadata[changeSet.name];
        // process references first
        for (const prop of Object.values(meta.properties)) {
            this.processReference(changeSet, prop);
        }
        // persist the entity itself
        await this.persistEntity(changeSet);
    }
    async persistEntity(changeSet) {
        const pk = this.metadata[changeSet.name].primaryKey;
        if (changeSet.delete) {
            await this.driver.nativeDelete(changeSet.name, changeSet.entity[pk]);
        }
        else if (changeSet.entity[pk]) {
            await this.driver.nativeUpdate(changeSet.name, changeSet.entity[pk], changeSet.payload);
        }
        else {
            changeSet.entity[pk] = await this.driver.nativeInsert(changeSet.name, changeSet.payload);
            this.identifierMap[changeSet.entity.uuid].setValue(changeSet.entity[pk]);
            delete changeSet.entity.__initialized;
        }
    }
    processReference(changeSet, prop) {
        const value = changeSet.payload[prop.name];
        if (value instanceof entity_1.EntityIdentifier) {
            changeSet.payload[prop.name] = value.getValue();
        }
        else if (Array.isArray(value) && value.some(item => item instanceof entity_1.EntityIdentifier)) {
            changeSet.payload[prop.name] = value.map(item => item instanceof entity_1.EntityIdentifier ? item.getValue() : item);
        }
        if (prop.onUpdate) {
            changeSet.entity[prop.name] = changeSet.payload[prop.name] = prop.onUpdate();
        }
    }
}
exports.ChangeSetPersister = ChangeSetPersister;
