import { IEntity, IEntityType, IPrimaryKey } from '../decorators';
import { EntityManager } from '../EntityManager';
import { FilterQuery } from '..';
export declare class UnitOfWork {
    private readonly em;
    /** map of references to managed entities */
    private readonly identityMap;
    /** holds copy of identity map so we can compute changes when persisting managed entities */
    private readonly originalEntityData;
    /** map of wrapped primary keys so we can compute change set without eager commit */
    private readonly identifierMap;
    private readonly persistStack;
    private readonly removeStack;
    private readonly changeSets;
    private readonly metadata;
    private readonly changeSetComputer;
    private readonly changeSetPersister;
    constructor(em: EntityManager);
    addToIdentityMap(entity: IEntity): void;
    getById<T extends IEntityType<T>>(entityName: string, id: IPrimaryKey): T;
    tryGetById<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T> | IPrimaryKey): T | null;
    getIdentityMap(): Record<string, IEntity>;
    persist<T extends IEntityType<T>>(entity: T, visited?: IEntity[]): void;
    remove(entity: IEntity, visited?: IEntity[]): void;
    commit(): Promise<void>;
    clear(): void;
    unsetIdentity(entity: IEntity): void;
    computeChangeSets(): void;
    private findNewEntities;
    private processReference;
    private commitChangeSet;
    private runHooks;
    /**
     * clean up persist/remove stack from previous persist/remove calls for this entity done before flushing
     */
    private cleanUpStack;
    private postCommitCleanup;
    private hasIdentifier;
    private cascade;
    private cascadeReference;
}
