import { NamingStrategy } from '../naming-strategy';
import { CacheAdapter, FileCacheAdapter } from '../cache';
import { TypeScriptMetadataProvider } from '../metadata';
import { EntityRepository } from '../entity';
import { MetadataProvider } from '../metadata';
import { EntityClass, EntityName, EntityOptions, IEntity } from '../decorators';
import { Hydrator, ObjectHydrator } from '../hydration';
import { EntityFactory } from '../entity';
import { Logger } from './Logger';
import { EntityManager } from '../EntityManager';
import { IDatabaseDriver } from '..';
export declare class Configuration {
    static readonly DEFAULTS: {
        entities: never[];
        entitiesDirs: never[];
        entitiesDirsTs: never[];
        tsConfigPath: string;
        autoFlush: boolean;
        strict: boolean;
        logger: () => undefined;
        baseDir: string;
        entityRepository: typeof EntityRepository;
        hydrator: typeof ObjectHydrator;
        debug: boolean;
        cache: {
            enabled: boolean;
            adapter: typeof FileCacheAdapter;
            options: {
                cacheDir: string;
            };
        };
        metadataProvider: typeof TypeScriptMetadataProvider;
    };
    private readonly options;
    private readonly logger;
    private readonly driver;
    private readonly cache;
    constructor(options: Options, validate?: boolean);
    get<T extends keyof MikroORMOptions, U>(key: T, defaultValue?: U): MikroORMOptions[T];
    getLogger(): Logger;
    getClientUrl(hidePassword?: boolean): string;
    getDriver(): IDatabaseDriver;
    getNamingStrategy(): NamingStrategy;
    getHydrator(factory: EntityFactory): Hydrator;
    getMetadataProvider(): MetadataProvider;
    getCacheAdapter(): CacheAdapter;
    getRepositoryClass(customRepository: EntityOptions['customRepository']): MikroORMOptions['entityRepository'];
    private init;
    private validateOptions;
    private initDriver;
    private cached;
}
export interface MikroORMOptions {
    dbName: string;
    entities: EntityClass<IEntity>[];
    entitiesDirs: string[];
    entitiesDirsTs: string[];
    tsConfigPath: string;
    autoFlush: boolean;
    driver?: {
        new (config: Configuration): IDatabaseDriver;
    };
    namingStrategy?: {
        new (): NamingStrategy;
    };
    hydrator: {
        new (factory: EntityFactory, driver: IDatabaseDriver): Hydrator;
    };
    entityRepository: {
        new (em: EntityManager, entityName: EntityName<IEntity>): EntityRepository<IEntity>;
    };
    clientUrl?: string;
    host?: string;
    port?: number;
    user?: string;
    password?: string;
    multipleStatements?: boolean;
    strict: boolean;
    logger: (message: string) => void;
    debug: boolean;
    baseDir: string;
    cache: {
        enabled: boolean;
        adapter: {
            new (...params: any[]): CacheAdapter;
        };
        options: Record<string, any>;
    };
    metadataProvider: {
        new (config: Configuration): MetadataProvider;
    };
}
export declare type Options = Pick<MikroORMOptions, Exclude<keyof MikroORMOptions, keyof typeof Configuration.DEFAULTS>> | MikroORMOptions;
